// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef IPC_IPC_MOJOM_H_
#define IPC_IPC_MOJOM_H_

#include <stdint.h>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/strings/string_piece.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map.h"
#include "mojo/public/cpp/bindings/message_filter.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/no_interface.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "ipc/ipc.mojom-internal.h"
#include "mojo/public/cpp/bindings/array.h"
#include "mojo/public/cpp/bindings/string.h"
namespace IPC {
namespace mojom {

class Channel;
using ChannelPtr = mojo::InterfacePtr<Channel>;
using ChannelPtrInfo = mojo::InterfacePtrInfo<Channel>;
using ChannelRequest = mojo::InterfaceRequest<Channel>;
using ChannelAssociatedPtr =
    mojo::AssociatedInterfacePtr<Channel>;
using ChannelAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<Channel>;
using ChannelAssociatedRequest =
    mojo::AssociatedInterfaceRequest<Channel>;

class Bootstrap;
using BootstrapPtr = mojo::InterfacePtr<Bootstrap>;
using BootstrapPtrInfo = mojo::InterfacePtrInfo<Bootstrap>;
using BootstrapRequest = mojo::InterfaceRequest<Bootstrap>;
using BootstrapAssociatedPtr =
    mojo::AssociatedInterfacePtr<Bootstrap>;
using BootstrapAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<Bootstrap>;
using BootstrapAssociatedRequest =
    mojo::AssociatedInterfaceRequest<Bootstrap>;

class SerializedHandle;
class SerializedHandleDataView;
using SerializedHandlePtr = mojo::StructPtr<SerializedHandle>;


class ChannelProxy;
class ChannelStub;

class ChannelRequestValidator;

class Channel {
 public:
  static const char Name_[];
  static const uint32_t Version_ = 0;
  static const bool PassesAssociatedKinds_ = false;
  static const bool HasSyncMethods_ = false;

  using Proxy_ = ChannelProxy;
  using Stub_ = ChannelStub;

  using RequestValidator_ = ChannelRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;

  virtual ~Channel() {}

  virtual void Receive(mojo::Array<uint8_t> data, mojo::Array<SerializedHandlePtr> handles) = 0;
};

class BootstrapProxy;
class BootstrapStub;

class BootstrapRequestValidator;
class BootstrapResponseValidator;

class Bootstrap {
 public:
  static const char Name_[];
  static const uint32_t Version_ = 0;
  static const bool PassesAssociatedKinds_ = true;
  static const bool HasSyncMethods_ = false;

  using Proxy_ = BootstrapProxy;
  using Stub_ = BootstrapStub;

  using RequestValidator_ = BootstrapRequestValidator;
  using ResponseValidator_ = BootstrapResponseValidator;

  virtual ~Bootstrap() {}


  using InitCallback = base::Callback<void(int32_t)>;
  virtual void Init(ChannelAssociatedRequest to_client_channel, ChannelAssociatedPtrInfo to_server_channel, int32_t pid, const InitCallback& callback) = 0;
};

class ChannelProxy
    : public Channel,
      public mojo::internal::ControlMessageProxy {
 public:
  explicit ChannelProxy(mojo::MessageReceiverWithResponder* receiver);
  void Receive(mojo::Array<uint8_t> data, mojo::Array<SerializedHandlePtr> handles) override;

  mojo::internal::SerializationContext* serialization_context() {
    return &serialization_context_;
  }

 private:
  mojo::internal::SerializationContext serialization_context_;
};

class BootstrapProxy
    : public Bootstrap,
      public mojo::internal::ControlMessageProxy {
 public:
  explicit BootstrapProxy(mojo::MessageReceiverWithResponder* receiver);
  void Init(ChannelAssociatedRequest to_client_channel, ChannelAssociatedPtrInfo to_server_channel, int32_t pid, const InitCallback& callback) override;

  mojo::internal::SerializationContext* serialization_context() {
    return &serialization_context_;
  }

 private:
  mojo::internal::SerializationContext serialization_context_;
};
class ChannelStub : public mojo::MessageReceiverWithResponderStatus {
 public:
  ChannelStub();
  ~ChannelStub() override;
  void set_sink(Channel* sink) { sink_ = sink; }
  Channel* sink() { return sink_; }
  mojo::internal::SerializationContext* serialization_context() {
    return &serialization_context_;
  }

  bool Accept(mojo::Message* message) override;
  bool AcceptWithResponder(mojo::Message* message,
      mojo::MessageReceiverWithStatus* responder) override;

 private:
  Channel* sink_;
  mojo::internal::SerializationContext serialization_context_;
  mojo::internal::ControlMessageHandler control_message_handler_;
};
class BootstrapStub : public mojo::MessageReceiverWithResponderStatus {
 public:
  BootstrapStub();
  ~BootstrapStub() override;
  void set_sink(Bootstrap* sink) { sink_ = sink; }
  Bootstrap* sink() { return sink_; }
  mojo::internal::SerializationContext* serialization_context() {
    return &serialization_context_;
  }

  bool Accept(mojo::Message* message) override;
  bool AcceptWithResponder(mojo::Message* message,
      mojo::MessageReceiverWithStatus* responder) override;

 private:
  Bootstrap* sink_;
  mojo::internal::SerializationContext serialization_context_;
  mojo::internal::ControlMessageHandler control_message_handler_;
};
class ChannelRequestValidator : public mojo::MessageFilter {
 public:
  explicit ChannelRequestValidator(mojo::MessageReceiver* sink = nullptr);

  bool Accept(mojo::Message* message) override;
};
class BootstrapRequestValidator : public mojo::MessageFilter {
 public:
  explicit BootstrapRequestValidator(mojo::MessageReceiver* sink = nullptr);

  bool Accept(mojo::Message* message) override;
};
class BootstrapResponseValidator : public mojo::MessageFilter {
 public:
  explicit BootstrapResponseValidator(mojo::MessageReceiver* sink = nullptr);

  bool Accept(mojo::Message* message) override;
};





class SerializedHandle {
 public:
  using DataView = SerializedHandleDataView;
  using Data_ = internal::SerializedHandle_Data;
  
  enum class Type : int32_t {
    MOJO_HANDLE,
    PLATFORM_FILE,
    WIN_HANDLE,
    MACH_PORT,
  };

  static SerializedHandlePtr New();

  template <typename U>
  static SerializedHandlePtr From(const U& u) {
    return mojo::TypeConverter<SerializedHandlePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, SerializedHandle>::Convert(*this);
  }

  SerializedHandle();
  ~SerializedHandle();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = SerializedHandlePtr>
  SerializedHandlePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, SerializedHandle>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static mojo::Array<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        SerializedHandlePtr, mojo::Array<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(mojo::Array<uint8_t> input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        SerializedHandlePtr, mojo::Array<uint8_t>>(
            std::move(input), output);
  }

  mojo::ScopedHandle the_handle;
  SerializedHandle::Type type;
};

class SerializedHandleDataView {
 public:
  SerializedHandleDataView(
      internal::SerializedHandle_Data* data,
      mojo::internal::SerializationContext* context);
  mojo::ScopedHandle TakeTheHandle();
  template <typename UserType>
  bool ReadType(UserType* value) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::IPC::mojom::SerializedHandle::Type>(
        data_value, value);
  }

  ::IPC::mojom::SerializedHandle::Type type() const;
 private:
  internal::SerializedHandle_Data* data_;
  mojo::internal::SerializationContext* context_;
};


template <typename StructPtrType>
SerializedHandlePtr SerializedHandle::Clone() const {
  // Use StructPtrType to prevent the compiler from trying to compile this
  // without being asked.
  StructPtrType rv(New());
  rv->the_handle = mojo::internal::Clone(the_handle);
  rv->type = mojo::internal::Clone(type);
  return rv;
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, SerializedHandle>::value>::type*>
bool SerializedHandle::Equals(const T& other) const {
  if (!mojo::internal::Equals(this->the_handle, other.the_handle))
    return false;
  if (!mojo::internal::Equals(this->type, other.type))
    return false;
  return true;
}

inline std::ostream& operator<<(std::ostream& os, SerializedHandle::Type value) {
  switch(value) {
    case SerializedHandle::Type::MOJO_HANDLE:
      return os << "SerializedHandle::Type::MOJO_HANDLE";
    case SerializedHandle::Type::PLATFORM_FILE:
      return os << "SerializedHandle::Type::PLATFORM_FILE";
    case SerializedHandle::Type::WIN_HANDLE:
      return os << "SerializedHandle::Type::WIN_HANDLE";
    case SerializedHandle::Type::MACH_PORT:
      return os << "SerializedHandle::Type::MACH_PORT";
    default:
      return os << "Unknown SerializedHandle::Type value: " << static_cast<int32_t>(value);
  }
}

inline bool IsKnownEnumValue(SerializedHandle::Type value) {
  return ::IPC::mojom::internal::SerializedHandle_Data::Type_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
}  // namespace mojom
}  // namespace IPC

namespace mojo {


template <>
struct EnumTraits<::IPC::mojom::SerializedHandle::Type, ::IPC::mojom::SerializedHandle::Type> {
  static ::IPC::mojom::SerializedHandle::Type ToMojom(::IPC::mojom::SerializedHandle::Type input) { return input; }
  static bool FromMojom(::IPC::mojom::SerializedHandle::Type input, ::IPC::mojom::SerializedHandle::Type* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::IPC::mojom::SerializedHandle::Type, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::IPC::mojom::SerializedHandle::Type, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::IPC::mojom::SerializedHandle::Type>(input), output);
  }
};

}  // namespace internal


template <>
struct StructTraits<::IPC::mojom::SerializedHandle, ::IPC::mojom::SerializedHandlePtr> {
  static bool IsNull(const ::IPC::mojom::SerializedHandlePtr& input) { return !input; }
  static void SetToNull(::IPC::mojom::SerializedHandlePtr* output) { output->reset(); }
  static decltype(::IPC::mojom::SerializedHandle::the_handle)& the_handle(
      ::IPC::mojom::SerializedHandlePtr& input) {
    return input->the_handle;
  }
  static decltype(::IPC::mojom::SerializedHandle::type) type(
      const ::IPC::mojom::SerializedHandlePtr& input) {
    return input->type;
  }

  static bool Read(::IPC::mojom::SerializedHandleDataView input, ::IPC::mojom::SerializedHandlePtr* output);
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::IPC::mojom::SerializedHandlePtr, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::IPC::mojom::SerializedHandle, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::IPC::mojom::internal::SerializedHandle_Data);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::IPC::mojom::internal::SerializedHandle_Data** output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input)) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
          ::IPC::mojom::internal::SerializedHandle_Data::New(buffer);
      mojo::internal::Serialize<mojo::ScopedHandle>(
          CallWithContext(Traits::the_handle, input, custom_context), &result->the_handle, context);
      MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
          !mojo::internal::IsHandleOrInterfaceValid(result->the_handle),
          mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
          "invalid the_handle in SerializedHandle struct");
      mojo::internal::Serialize<::IPC::mojom::SerializedHandle::Type>(
          CallWithContext(Traits::type, input, custom_context), &result->type);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::IPC::mojom::internal::SerializedHandle_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::IPC::mojom::SerializedHandleDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo

#endif  // IPC_IPC_MOJOM_H_